unit MacWin;

{ MacWin (C)2K By Paul TOTH <tothpaul@free.fr>
 http://tothpaul.free.fr
}

{
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type
  TWMSizing=record
   Msg:Cardinal;
   wParam:Word;
   Rect:^TRect;
   Result:longint;
  end;

  TLocation=(wNone,wTitle,wClose,wGrow,wVScroll,wHScroll,wUp,wDown,wLeft,wRight,wSize);

  TMWFlags=set of (fClose,fClosing,fGrow,fGrowing,fScroll,fVBar,fSizeBox);
  TMWOptions=set of (mwCloseButton,mwGrowButton,mwVScroll,mwHScroll,mwSizeButton);

  TMacWin = class(TCustomControl)
  private
   fActive:boolean;
   fWhere :TLocation;
   fFlags :TMWFlags;
   fOptions:TMWOptions;
   fNCPaint:integer;
   fReduice:TRect;
   fTitleFont:TFont;
   fTitleSize:integer;
   fBorderStyle:TBorderStyle;
   EOnPaint:TNotifyEvent;
  // ScrollBars
   fTopPos:integer;
   fLeftPos:integer;
   fVertRange:integer;
   fHorzRange:integer;
   fScrollWhat:TLocation;
   fScrollX:integer;
   fScrollY:integer;
   fVertLen:integer;
   fHorzLen:integer;
   fVertBar:TRect;
   fHorzBar:TRect;
   fVertTrack:TRect;
   fHorzTrack:TRect;
   procedure SetActive(Value:Boolean);
   procedure SetOptions(Value:TMWOptions);
   procedure SetTitleFont(Value:TFont);
   procedure TitleFontChange(Sender:TObject);
   procedure SetBorderStyle(Value:TBorderStyle);
   procedure WMNCHitTest(Var Msg:TWMNCHitTest); message WM_NCHITTEST;
   procedure WMNCCalcSize(Var Msg:TWMNCCalcSize); message WM_NCCALCSIZE;
   procedure WMGetMinMaxInfo(Var Msg:TWMGetMinMaxInfo); message WM_GETMINMAXINFO;
   procedure WMNCPaint(Var Msg:TMessage); message WM_NCPAINT;
   procedure WMSize(Var Msg:TMessage); message WM_SIZE;
   procedure WMMouseActivate(Var Msg:TMessage); message WM_MOUSEACTIVATE;
   procedure CMTextChanged(Var Msg:TMessage); message CM_TEXTCHANGED;
   procedure WMNCLButtonDown(Var Msg:TWMNCLButtonDown); message WM_NCLBUTTONDOWN;
   procedure WMMouseMove(Var Msg:TWMMouseMove); message WM_MOUSEMOVE;
   procedure WMLButtonUp(Var Msg:TWMLButtonUp); message WM_LBUTTONUP;
   procedure WMSetCursor(Var Msg:TWMSetCursor); message WM_SETCURSOR;
   procedure CMControlListChange(Var Msg:TCMControlListChange); message CM_CONTROLLISTCHANGE;
  // ScrollBars
   procedure SetTopPos(Value:integer);
   procedure SetLeftPos(Value:integer);
  protected
   constructor Create(AOwner:TComponent); override;
   destructor Destroy; override;
   procedure Arrow(Index,x,y:integer);
   function  NCBegin:boolean;
   procedure NCDone;
   procedure NCClose;
   procedure NCGrow;
   procedure NCTitle;
   procedure NCVertBar;
   procedure NCHorzBar;
   procedure NCPaint;
   procedure ScrollRange;
   procedure Paint; override;
  public
   property Canvas;
  published
   property Active:boolean read fActive write SetActive;
   property TitleFont:TFont read fTitleFont write SetTitleFont;
   Property BorderStyle:TBorderStyle read fBorderStyle write SetBorderStyle;
   property OnPaint:TNotifyEvent read EOnPaint write EOnPaint;
   property Options:TMWOptions read fOptions write SetOptions;
   property Color default clWhite;
   property ParentFont default true;
   property Ctl3D default false;
   property ParentCtl3D default false;
   property Align;
   property Caption;
   property Font;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('MySoft.Fun', [TMacWin]);
end;

Const
 SB_HEIGHT=16;
 SB_WIDTH =16;

{$R MACRES}

Var
 ActiveMac:TMacWin=nil;
 ScrollBrush:TBitmap;
 CloseButton:TBitmap;
 GrowButton :TBitmap;
 ClickButton:TBitmap;
 cMacArrow:HCursor;


Constructor TMacWin.Create(AOwner:TComponent);
 begin
  inherited Create(AOwner);
  fOptions:=[mwCloseButton,mwGrowButton,mwVScroll,mwHScroll,mwSizeButton];
  fFlags:=[fVBar,fSizeBox];
  fNCPaint:=0;
  fTitleFont:=TFont.Create;
  fTitleFont.Name:='Chicago';
  fTitleFont.OnChange:=TitleFontChange;
  fTitleSize:=8-fTitleFont.Height;
  Font.Name:='Geneva';
  ControlStyle:=ControlStyle +
   [csAcceptsControls, csCaptureMouse, csDesignInteractive, csClickEvents, csOpaque, csDoubleClicks];
  Color:=clWhite;
  ParentCtl3D:=False;
  ParentFont:=True;
  Ctl3D:=False;
  fActive:=False;

  fTopPos:=0;
  fLeftPos:=0;
  fVertRange:=0;
  fHorzRange:=0;

  Cursor:=1;
 end;

Destructor TMacWin.Destroy;
 begin
  if ActiveMac=Self then ActiveMac:=nil;
  fTitleFont.Free;
  inherited Destroy;
 end;

procedure TMacWin.SetActive(Value:boolean);
 begin
  if fActive<>Value then begin
   fActive:=Value;
   if fActive then begin
    if ActiveMac<>nil then ActiveMac.Active:=False;
    ActiveMac:=Self;
    SetZOrder(True);
   end;
   NCTitle;
  end;
 end;

procedure TMacWin.SetOptions(Value:TMWOptions);
 var
  O:TMWOptions;
 begin
  if Value<>fOptions then begin
   fOptions:=Value;
   fFlags:=fFlags-[fVBar,fSizeBox];
   O:=[mwVScroll,mwHScroll,mwSizeButton] * fOptions; // What is in fOptions ?
   if (O<>[]) then begin
    if (O<>[mwVScroll])and(O<>[mwHScroll]) then fFlags:=fFlags+[fSizeBox];
    if (mwVScroll in O)or(O=[mwSizeButton]) then fFlags:=fFlags+[fVBar];
   end;
   RecreateWnd;
  end;
 end;

procedure TMacWin.SetTitleFont(Value:TFont);
 begin
  fTitleFont.Assign(Value);
 end;

procedure TMacWin.TitleFontChange(Sender:TObject);
 var
  NewSize:integer;
 begin
  NewSize:=8-fTitleFont.Height;
  if NewSize=fTitleSize then
   NCTitle
  else begin
   fTitleSize:=NewSize;
   RecreateWnd; // force a WM_NCCALCSIZE message
  end;
 end;

procedure TMacWin.SetBorderStyle(Value: TBorderStyle);
begin
  if FBorderStyle <> Value then begin
   FBorderStyle := Value;
   RecreateWnd;
  end;
end;

procedure TMacWin.WMNCHitTest(Var Msg:TWMNCHitTest);
 var
  Mouse:TPoint;
  vs,hs:boolean;
  ButtonY:integer;
 begin
  inherited;
  Mouse:=ScreenToClient(Point(Msg.XPos,Msg.YPos));
  if fScroll in fFlags then
   fWhere:=fScrollWhat
  else begin
   fWhere:=wNone;
   if Mouse.Y<0 then begin
    fWhere:=wTitle;
    Msg.Result:=HTCAPTION;
    ButtonY:=(fTitleSize-10) div 2-fTitleSize;
    if (mwCloseButton in fOptions)and(Mouse.Y>ButtonY)and(Mouse.Y<ButtonY+10) then begin
     if (Mouse.X>8)and(Mouse.X<18) then begin
      fWhere:=wClose;
      Msg.Result:=HTBORDER;
     end else
     if (mwGrowButton in fOptions)and(Mouse.X>Width-20)and(Mouse.X<Width-20+11) then begin
      fWhere:=wGrow;
      Msg.Result:=HTBORDER;
     end;
    end;
   end else begin
    inc(Mouse.Y,fTitleSize);
    vs:=(Mouse.X>Width-SB_WIDTH);
    hs:=(Mouse.Y>Height-SB_HEIGHT);
    if vs then begin
     if hs and (mwSizeButton in fOptions) then begin
      fWhere:=wSize;
      Msg.result:=HTBOTTOMRIGHT;
     end else
     if (fVertRange>0)and(mwVScroll in fOptions) then begin
      if Mouse.Y<fVertBar.Top+SB_WIDTH    then fWhere:=wUp else
      if Mouse.Y>fVertBar.Bottom-SB_WIDTH then fWhere:=wDown else
      if (Mouse.Y>fVertTrack.Top)and(Mouse.Y<fVertTrack.Bottom) then fWhere:=wVScroll;
      Msg.Result:=HTBORDER;
     end;
    end else
    if hs and (fHorzRange>0)and(mwHScroll in fOptions) then begin
     if Mouse.X<fHorzBar.Left+SB_HEIGHT  then fWhere:=wLeft  else
     if Mouse.X>fHorzBar.Right-SB_HEIGHT then fWhere:=wRight else
     if (Mouse.X>fHorzTrack.Left)and(Mouse.X<fHorzTrack.Right) then fWhere:=wHScroll;
     Msg.Result:=HTBORDER;
    end;
   end;
  end;
 end;

procedure TMacWin.WMNCCalcSize(Var Msg:TWMNCCalcSize);
 begin
  inherited;
  with Msg do begin
   {if CalcValidRects then} begin
    with CalcSize_Params^.rgrc[0] do begin
     Inc(Top,fTitleSize);
     Inc(Left);
     if fVBar in fFlags then Dec(Right,SB_WIDTH) else Dec(Right);
     if (mwHScroll in Options) then Dec(Bottom,SB_HEIGHT) else Dec(Bottom);
    end;
   end;
  end;
 end;

procedure TMacWin.WMGetMinMaxInfo(Var Msg:TWMGetMinMaxInfo);
 begin
  inherited;
  with Msg.MinMaxInfo^ do begin
   Canvas.Font:=TitleFont;
   ptMinTrackSize.x:=Canvas.TextWidth(Caption)+SB_WIDTH+2*20+4;
   ptMinTrackSize.y:=fTitleSize+SB_HEIGHT;
  end;
 end;

function TMacWin.NCBegin:boolean;
 begin
  if fNCPaint=0 then begin
   Result:=False;
   if Parent=nil then exit;
   if Not Visible then exit;
   Canvas.Handle:=GetWindowDC(Self.Handle);
  end;
  Inc(fNCPaint);
  Result:=True;
 end;

procedure TMacWin.NCDone;
 begin
  if fNCPaint>0 then begin
   dec(fNCPaint);
   if (fNCPaint=0) then Canvas.Handle:=0;
  end else begin
   raise Exception.Create('More NCDones then NCBegins');
  end;
 end;

procedure TMacWin.NCClose;
 var
  cy:integer;
 begin
  if not (mwCloseButton in Options) then exit;
  if not NCBegin then exit;
  cy:=1+(fTitleSize-6*2) div 2;
  if (fClosing in fFlags) then
   Canvas.Draw(8,cy,ClickButton)
  else
   Canvas.Draw(8,cy,CloseButton);
  NCDone;
 end;

procedure TMacWin.NCGrow;
 var
  cx,cy:integer;
 begin
  if not (mwGrowButton in Options) then exit;
  if not NCBegin then exit;
  cx:=Width-21;
  cy:=1+(fTitleSize-6*2) div 2;
  if (fGrowing in fFlags) then
   Canvas.Draw(cx,cy,ClickButton)
  else
   Canvas.Draw(cx,cy,GrowButton);
  NCDone;
 end;

procedure TMacWin.NCTitle;
 var
  cx,cy:integer;
  cs:string;
 begin
  if not NCBegin then exit;
  With Canvas do begin
  // Title Bar
   Pen.Color:=clBlack;
   Brush.Style:=bsSolid;
   Brush.Color:=clWhite;
   Rectangle(0,0,Width,fTitleSize);
  // Focus
   if fActive then begin
    cy:=1+(fTitleSize-6*2) div 2;
  // Title Lines
    for cx:=1 to 6 do begin
     MoveTo(2,cy); LineTo(Width-2,cy);
     inc(cy,2);
    end;
   // CloseButton
    NCClose;
   // StateButton
    NCGrow;
   end;
  // Title Caption
   Font:=fTitleFont;
   cs:=' '+Caption+' ';
   cx:=(Width-TextWidth(cs)) div 2;
   cy:=(fTitleSize+Font.Height-4) div 2;
   TextOut(cx,cy,cs);
  end;
  // Release Handle
  NCDone;
 end;

procedure TMacWin.Arrow(Index,x,y:integer);
 Type
  TArrow=array[0..6,0..1] of integer;
 Const
  Arrows:array[0..3] of TArrow=(
   { up }    (( 7, 2),(13, 8),(10, 8),(10,12),( 4,12),( 4, 8),( 1, 8)),
   { down }  (( 4, 3),(10, 3),(10, 7),(13, 7),( 7,13),( 1, 7),( 4, 7)),
   { left }  (( 7, 1),( 7, 4),(11, 4),(11,10),( 7,10),( 7,13),( 1, 7)),
   { right } (( 8, 1),(14, 7),( 8,13),( 8,10),( 4,10),( 4, 4),( 8, 4))
  );
 var
  i:integer;
  a:^TArrow;
 begin
  a:=@Arrows[Index];
  with Canvas do begin
   Rectangle(x,y,x+SB_WIDTH,y+SB_HEIGHT);
   MoveTo(x+a[6,0],y+a[6,1]);
   for i:=0 to 6 do LineTo(x+a[i,0],y+a[i,1]);
  end;
 end;

procedure TMacWin.NCVertBar;
 begin
  if not NCBegin then exit;
  with Canvas do begin
   Brush.Style:=bsSolid;
   Brush.Color:=clWhite;
   with fVertBar do begin
    Brush.Bitmap:=ScrollBrush;
    Rectangle(Left,Top+SB_WIDTH,Right,Bottom-SB_WIDTH);
    Brush.Bitmap:=nil;

    fVertLen:=fVertBar.Bottom-fVertBar.Top-3*SB_WIDTH;
    fVertTrack.Left  :=Left+1;
    fVertTrack.Right :=Right-1;
    fVertTrack.Top   :=Top+SB_WIDTH+(fTopPos*fVertLen) div fVertRange;
    fVertTrack.Bottom:=fVertTrack.Top+SB_WIDTH;

    with fVertTrack do Rectangle(Left,Top,Right,Bottom);
   end;
  end;
  NCDone;
 end;


procedure TMacWin.NCHorzBar;
 begin
  if not NCBegin then exit;
  with Canvas do begin
   with fHorzBar do begin
    Brush.Bitmap:=ScrollBrush;
    Rectangle(Left+SB_HEIGHT,Top,Right-SB_HEIGHT,Bottom);
    Brush.Bitmap:=nil;
    fHorzLen:=Right-Left-3*SB_WIDTH;
    fHorzTrack.Top   :=Top+1;
    fHorzTrack.Bottom:=Bottom-1;
    fHorzTrack.Left  :=Left+SB_HEIGHT+(fLeftPos*fHorzLen) div fHorzRange;
    fHorzTrack.Right :=fHorzTrack.Left+SB_HEIGHT;
    with fHorzTrack do Rectangle(Left,Top,Right,Bottom);
   end;
  end;
  NCDone;
 end;

procedure TMacWin.NCPaint;
 begin
  if not NCBegin then exit;
  With Canvas do begin
   NCTitle;
  // Border
   Pen.Color:=clBlack;
   Brush.Style:=bsClear;
   Rectangle(0,0,Width,Height);
  // ScrollBars
   Brush.Style:=bsSolid;
   if fVBar in fFlags then
   with fVertBar do begin
    if fVertRange>0 then begin
     Arrow(0,Left,Top);
     Arrow(1,Left,Bottom-SB_WIDTH);
     NCVertBar;
    end else
     Rectangle(Left,Top,Width,Height);
   end;
   if mwHScroll in fOptions then
   with fHorzBar do begin
    if fHorzRange>0 then begin
     Arrow(2,Left,Top);
     Arrow(3,Right-SB_HEIGHT,Top);
     NCHorzBar;
    end else
     Rectangle(Left,Top,Width,Height);
   end;
  // SizeButton
   if fSizeBox in fFlags then Rectangle(Width-SB_WIDTH,  Height-SB_HEIGHT,  Width,  Height);
   if mwSizeButton in fOptions then begin
    Rectangle(Width-SB_WIDTH+5,Height-SB_HEIGHT+5,Width-2,Height-2);
    Rectangle(Width-SB_WIDTH+3,Height-SB_HEIGHT+3,Width-6,Height-6);
   end;
  end;
 // Release Handle
  NCDone;
 end;

Procedure TMacWin.ScrollRange;
 var
  t,l,b,r:integer;
  i:integer;
 begin
  t:=0;
  l:=0;
  b:=0;//ClientHeight;
  r:=0;//ClientWidth;
  for i:=0 to ControlCount-1 do begin
   with Controls[i] do begin
    if Top<t then t:=Top;
    if Left<l then l:=Left;
    if Left+Width>r then r:=Left+Width;
    if Top+Height>b then b:=Top+Height;
   end;
  end;
  fHorzRange:=r-l-ClientWidth;  if fHorzRange<0 then fHorzRange:=0;
  fVertRange:=b-t-ClientHeight; if fVertRange<0 then fVertRange:=0;

  fVertBar:=Rect(Width-SB_WIDTH,fTitleSize-1,Width,Height);
  if ([mwVScroll,mwHScroll,mwSizeButton]*fOptions <> [mwVScroll]) then Dec(fVertBar.Bottom,SB_HEIGHT-1);
  fHorzBar:=Rect(0,fVertBar.Bottom-1,Width,Height);
  if ([mwVScroll,mwHScroll,mwSizeButton]*fOptions <> [mwHScroll]) then Dec(fHorzBar.Right,SB_WIDTH-1);

  if fTopPos>fVertRange then SetTopPos(fVertRange);
  if fLeftPos>fHorzRange then SetLeftPos(fHorzRange);
 end;

Procedure TMacWin.WMNCPaint(Var Msg:TMessage);
 begin
//  inherited;
  NCPaint;
 end;

Procedure TMacWin.WMSize(Var Msg:TMessage);
 begin
  inherited;
  ScrollRange;
  NCPaint;
  //NCPaint;
 end;

procedure TMacWin.WMMouseActivate(Var Msg:TMessage);
 begin
  inherited;
  Active:=True;
 end;

procedure TMacWin.CMTextChanged(var Msg: TMessage);
 begin
  inherited;
  NCTitle;
 end;

procedure TMacWin.WMNCLButtonDown(Var Msg:TWMNCLButtonDown);
 Var
  Mouse:TPoint;
 begin
  inherited;
  Mouse:=ScreenToClient(Point(Msg.XCursor,Msg.YCursor));
//  Caption:='click #'+IntToStr(Msg.HitTest);
  case fWhere of
   wClose : begin
    Include(fFlags,fClose);
    Include(fFlags,fClosing);
    SetCaptureControl(Self);
    NCClose;
   end;
   wGrow : begin
    Include(fFlags,fGrow);
    Include(fFlags,fGrowing);
    SetCaptureControl(Self);
    NCGrow;
   end;
   wUp     : begin SetTopPos(fTopPos-(ClientHeight div 20)); NCVertBar; end;
   wDown   : begin SetTopPos(fTopPos+(ClientHeight div 20)); NCVertBar; end;
   wVScroll: Include(fFlags,fScroll);
   wLeft   : begin SetLeftPos(fLeftPos-(ClientWidth div 20)); NCHorzBar; end;
   wRight  : begin SetLeftPos(fLeftPos+(ClientWidth div 20)); NCHorzBar; end;
   wHScroll: Include(fFlags,fScroll);
  end;
  if fScroll in fFlags then begin
   fScrollWhat:=fWhere;
   fScrollX:=Mouse.X-fHorzTrack.Left+SB_HEIGHT;
   fScrollY:=Mouse.Y-fVertTrack.Top+SB_WIDTH+fTitleSize;
   SetCaptureControl(Self);
  end;
 end;

procedure TMacWin.WMMouseMove(Var Msg:TWMMouseMove);
 begin
  inherited;
  if fScroll in fFlags then begin
   case fScrollWhat of
    wVScroll:
     if fVertRange>0 then begin
      SetTopPos (((Msg.YPos-fScrollY)*fVertRange) div fVertLen);
      NCVertBar;
     end;
    wHScroll:
     if fHorzRange>0 then begin
      SetLeftPos(((Msg.XPos-fScrollX)*fHorzRange) div fHorzLen);
      NCHorzBar;
     end;
   end;
  end else
  if fClose in fFlags then begin
   if (fClosing in fFlags)and(fWhere<>wClose) then begin
    Exclude(fFlags,fClosing);
    NCClose;
   end else
   if (fWhere=wClose)and not(fClosing in fFlags) then begin
    Include(fFlags,fClosing);
    NCClose;
   end;
  end;
  if fGrow in fFlags then begin
   if (fGrowing in fFlags)and(fWhere<>wGrow) then begin
    Exclude(fFlags,fGrowing);
    NCGrow;
   end else
   if (fWhere=wGrow)and not(fGrowing in fFlags) then begin
    Include(fFlags,fGrowing);
    NCGrow;
   end;
  end;
 end;

procedure TMacWin.WMLButtonUp(Var Msg:TWMLButtonUp);
 begin
  inherited;
  if fScroll in fFlags then begin
   Exclude(fFlags,fScroll);
   SetCaptureControl(nil);
  end;
  if fClose in fFlags then begin
   Exclude(fFlags,fClose);
   SetCaptureControl(nil);
   if fWhere=wClose then Free else NCClose;
  end;
  if fGrow in fFlags then begin
   Exclude(fFlags,fGrow);
   Exclude(fFlags,fGrowing);
   NCGrow;
   SetCaptureControl(nil);
   if fWhere=wGrow then begin
    if Align=alClient then begin
     Align:=alNone;
     with fReduice do SetBounds(Left,Top,Right-Left,Bottom-Top)
    end else begin
     fReduice:=BoundsRect;
     Align:=alClient;
    end;
   end;
  end;
 end;

procedure TMacWin.WMSetCursor(Var Msg:TWMSetCursor);
 begin
  SetCursor(cMacArrow);
  Msg.Result:=1;
 end;

procedure TMacWin.CMControlListChange(Var Msg:TCMControlListChange);
 begin
  if Msg.Control<>nil then inherited;
  if Msg.Inserting then
   PostMessage(Handle,CM_CONTROLLISTCHANGE,0,0) // Repost message ...
  else begin
   ScrollRange; // ...compute Scroll Range (with new control inserted !)
   NCPaint;
  end;
 end;

Procedure TMacWin.Paint;
 begin
  with Canvas do begin
   Brush.Color:=Self.Color;
   Brush.Style:=bsSolid;
   FillRect(ClientRect);
  end;
  if Assigned(EOnPaint) then EOnPaint(Self);
 end;

procedure TMacWin.SetTopPos(Value:integer);
 begin
  if (Value<0) then Value:=0 else
  if (Value>fVertRange) then Value:=fVertRange;
  if fTopPos=Value then exit;
  ScrollBy(0,fTopPos-Value);
  fTopPos:=Value;
 end;

procedure TMacWin.SetLeftPos(Value:integer);
 begin
  if (Value<0) then Value:=0 else
  if (Value>fHorzRange) then Value:=fHorzRange;
  if fLeftPos=Value then exit;
  ScrollBy(fLeftPos-Value,0);
  fLeftPos:=Value;
 end;

function GetBitmap(Name:string):TBitmap;
 begin
  Result:=TBitmap.Create;
  Result.LoadFromResourceName(hInstance,Name);
 end;

initialization
 ScrollBrush:=GetBitmap('ScrollBrush');
 CloseButton:=GetBitmap('CLOSE');
 GrowButton :=GetBitmap('GROW');
 ClickButton:=GetBitmap('CLICK');
 cMacArrow:=LoadCursor(hInstance,'MACARROW');
// Screen.Cursors[2]:=LoadCursor(hInstance,'MACWAIT');

finalization
 ScrollBrush.Free;
 CloseButton.Free;
 GrowButton.Free;
 ClickButton.Free;
 DestroyCursor(cMacArrow);

end.
